"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
from dataclasses import dataclass, asdict
from typing import Dict, Optional


@dataclass
class MonsterDrop:
    """This class defines the drops of an OSRS monster.

    The MonsterDrop class is the object that retains all drop properties related
    to items dropped by a specific monster. This includes item properties (id,
    name) and drop properties (quantity, rarity, and drop requirements).
    """
    id: int = None
    name: str = None
    members: str = None
    quantity: str = None
    noted: bool = None
    rarity: str = None
    drop_requirements: Optional[str] = None

    def construct_json(self) -> Dict:
        """Construct dictionary/JSON of drop entry in a list for exporting or printing.

        :return json_out: A dictionary of a single drop instance.
        """
        return asdict(self)



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Description:
Tests for module: osrsbox.items_api.all_items

Copyright (c) 2020, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
import os
from pathlib import Path

from osrsbox.items_api import all_items

# The current number of items being loaded from the db
NUMBER_OF_ITEMS = 23513


def test_all_items_load_items_json(path_to_docs_dir: Path):
    path_to_items_json_dir_no_slash = path_to_docs_dir / "items-json"
    path_to_items_json_dir_slash = os.path.join(path_to_docs_dir, "items-json", "")

    for path in (path_to_items_json_dir_slash, path_to_items_json_dir_no_slash):
        all_db_items = all_items.AllItems(str(path))
        assert len(all_db_items.all_items) == NUMBER_OF_ITEMS


def test_all_items_load_items_complete(path_to_docs_dir: Path):
    path_to_items_complete = path_to_docs_dir / "items-complete.json"

    all_db_items = all_items.AllItems(str(path_to_items_complete))
    assert len(all_db_items.all_items) == NUMBER_OF_ITEMS



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2020, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
import json
from pathlib import Path
from dataclasses import asdict
from dataclasses import dataclass
from typing import Dict


@dataclass
class PrayerProperties:
    """This class defines the object structure and properties for an OSRS prayer.

    The PrayerProperties class is the object that retains all properties and stats
    for one specific prayer.
    """
    id: int = None
    name: str = None
    members: bool = None
    description: str = None
    drain_per_minute: float = None
    wiki_url: str = None
    requirements: Dict = None
    bonuses: Dict = None
    icon: str = None

    @classmethod
    def from_json(cls, json_dict: Dict) -> 'PrayerProperties':
        """Construct PrayerProperties object from dictionary/JSON."""
        return cls(**json_dict)

    def construct_json(self) -> Dict:
        """Construct dictionary for exporting or printing.

        :return: All class attributes stored in a dictionary.
        """
        return asdict(self)

    def export_json(self, pretty: bool, export_path: str):
        """Output PrayerProperties to JSON file.

        :param pretty: Toggles pretty (indented) JSON output.
        :param export_path: The folder location to save the JSON output to.
        """
        json_out = self.construct_json()
        out_file_name = str(self.id) + ".json"
        out_file_path = Path(export_path / out_file_name)
        with open(out_file_path, "w") as out_file:
            if pretty:
                json.dump(json_out, out_file, indent=4)
            else:
                json.dump(json_out, out_file)



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""

from dataclasses import dataclass, asdict
from typing import List, Dict


@dataclass
class ItemWeapon:
    """This class defines the properties for an equipable OSRS item that is a weapon.

    The ItemWeapon class is the object that retains all items properties related
    to equipable items that are weapons. This includes weapon attack speed,
    weapon type, stance, experience, and bonuses.
    """
    attack_speed: int
    weapon_type: str
    stances: List

    def construct_json(self) -> Dict:
        """Construct dictionary/JSON of ItemWeapon class for exporting or printing.

        :return: All class attributes stored in a dictionary.
        """
        return asdict(self)



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""

import json
from pathlib import Path
from typing import Dict, List, Union, Generator

from osrsbox.items_api.item_properties import ItemProperties

PATH_TO_ITEMS_COMPLETE_JSON = Path(__file__).absolute().parent / ".." / ".." / "docs" / "items-complete.json"
if not PATH_TO_ITEMS_COMPLETE_JSON.is_file():
    PATH_TO_ITEMS_COMPLETE_JSON = Path(__file__).absolute().parent / ".." / "docs" / "items-complete.json"
    if not PATH_TO_ITEMS_COMPLETE_JSON.is_file():
        raise ValueError("Error: Default item database file not found. Exiting")


class AllItems:
    """This class handles loading of the osrsbox-db items database.

    :param input_data_file_or_directory: The osrsbox-db items folder of JSON files, or single JSON file.
    """
    def __init__(self, input_data_file_or_directory: Path = PATH_TO_ITEMS_COMPLETE_JSON):
        self.all_items: List[ItemProperties] = list()
        self.all_items_dict: Dict[int, ItemProperties] = dict()
        self.load_all_items(input_data_file_or_directory)

    def __iter__(self) -> Generator[ItemProperties, None, None]:
        """Iterate (loop) over each ItemProperties object."""
        for item in self.all_items:
            yield item

    def __getitem__(self, id_number: int) -> ItemProperties:
        """Return the item definition object for a loaded item.

        :param id_number: The item ID number.
        :return: The item definition object linked to a specific ID number.
        """
        return self.all_items_dict[id_number]

    def __len__(self) -> int:
        """Return the count of the total number of items.

        :return: The total number of items.
        """
        return len(self.all_items)

    def lookup_by_item_id(self, item_id_number: int) -> ItemProperties:
        """Lookup a specific item ID and get the associated ItemProperties object.

        :param item_id_number: The item ID to lookup.
        :return: The ItemProperties object found from the lookup.
        :raises: KeyError when the item ID cannot be found.
        """
        try:
            item_properties = self.all_items_dict[item_id_number]
        except KeyError:
            raise KeyError("Cannot find the provided item ID number...")
        return item_properties

    def lookup_by_item_name(self, item_name: str, use_wiki_name: bool = False) -> ItemProperties:
        """Lookup a specific item name and get the associated ItemProperties object.

        This function performs a lookup on all items in the database. The property
        that is queried is the item name, or the `wiki_name` if specified in the
        method parameters. The only transformation performed on the item name
        provided is to convert it to lower case to (slightly) improve lookup recall.
        This function works on a first-come-first-served basis. The first instance
        where the name matches is returned.

        :param item_name: The item name to lookup.
        :param use_wiki_name: Whether to use the `wiki_name` instead of `name`.
        :return: The ItemProperties object found from the lookup.
        :raises: ValueError when the item name cannot be found.
        """
        # Set the property value
        lookup_property = "name"
        if use_wiki_name:
            lookup_property = "wiki_name"

        for item in self.all_items:
            # Get the raw property value
            name_value = getattr(item, lookup_property)
            # Check property value for None (only effective for wiki_name)
            if not name_value:
                continue

            # Check for an exact name match
            if name_value.lower() == item_name.lower():
                return item

        raise ValueError("Cannot find the provided item name...")

    def search_item_names(self, keyword: str) -> List[ItemProperties]:
        """Keyword search items and get the a list of ItemProperties objects.

        This function performs a search of all item names in the database. The name
        and wiki_name properties are searched. Results are returned as a list of
        ItemProperties objects. The only transformation performed is converting the
        search keyword and item name/wiki_name to lower case.

        :param keyword: The keyword to search for.
        :return: A list of ItemProperties objects found from the keyword search.
        :raises: ValueError when no keyword matches can be found.
        """
        item_results = list()
        for item in self.all_items:
            if keyword.lower() in item.name.lower():
                item_results.append(item)
            elif item.wiki_name:
                if keyword.lower() in item.wiki_name.lower():
                    item_results.append(item)

        return item_results

    def load_all_items(self, input_data_file_or_directory: Union[Path, str]) -> None:
        """Load the items database via a JSON file, or directory of JSON files.

        :param input_data_file_or_directory: The path to the data input.
        :raises ValueError: Valid input not found.
        """
        # Check if a str is supplied, if so, convert to Path object
        if isinstance(input_data_file_or_directory, str):
            input_data_file_or_directory = Path(input_data_file_or_directory)

        # Process the directory of JSON, or a single JSON file
        if input_data_file_or_directory.is_dir():
            self._load_items_from_directory(path_to_directory=input_data_file_or_directory)
        elif input_data_file_or_directory.is_file():
            self._load_items_from_file(path_to_json_file=input_data_file_or_directory)
        else:
            raise ValueError("Error: Valid input not found. Exiting.")

        # Sort the list of items
        self.all_items.sort(key=lambda x: x.id)

    def _load_items_from_directory(self, path_to_directory: Path) -> None:
        """Load item database from a directory of JSON files (`items-json`).

        :param path_to_directory: The path to the `items-json` directory.
        :raises ValueError: No JSON files found in supplied directory.
        """
        # Fetch all .json files in provided dir
        json_files = list(path_to_directory.glob("*.json"))

        try:
            json_files[0]
        except IndexError as e:
            raise ValueError("Error: No files found in directory, check the supplied path. Exiting.") from e

        # Loop through every item in JSON file
        for json_file in json_files:
            with open(json_file) as input_json_file:
                temp = json.load(input_json_file)

            self._load_item(temp)

    def _load_items_from_file(self, path_to_json_file: Path) -> None:
        """Load item database from a single JSON file (`items-complete.json`).

        :param path_to_json_file: The path to the `items-complete.json` file.
        """
        with open(path_to_json_file) as input_json_file:
            temp = json.load(input_json_file)

        for entry in temp:
            self._load_item(temp[entry])

    def _load_item(self, item_json: Dict) -> None:
        """Convert the `item_json` into a :class:`ItemProperties` and store it.

        :param item_json: A dict from an open and loaded JSON file.
        :raises ValueError: Cannot populate item.
        """
        # Load the item using the ItemProperties class
        try:
            item_def = ItemProperties.from_json(item_json)
        except TypeError as e:
            raise ValueError("Error: Invalid JSON structure found, check supplied input. Exiting") from e

        # Add item to list
        self.all_items.append(item_def)
        self.all_items_dict[item_def.id] = item_def



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
import json
from pathlib import Path
from dataclasses import asdict
from dataclasses import dataclass
from typing import Dict
from typing import Optional

from osrsbox.items_api.item_equipment import ItemEquipment
from osrsbox.items_api.item_weapon import ItemWeapon


@dataclass
class ItemProperties:
    """This class defines the object structure and properties for an OSRS item.

    The ItemProperties class is the object that retains all properties and stats
    for one specific item. Every item has the properties defined in this class.
    Equipable items have additional properties defined in the linked ItemEquipment
    class.
    """
    id: int
    name: str
    incomplete: bool
    members: bool
    tradeable: Optional[bool]
    tradeable_on_ge: bool
    stackable: bool
    stacked: bool
    noted: bool
    noteable: bool
    linked_id_item: Optional[int]
    linked_id_noted: Optional[int]
    linked_id_placeholder: Optional[int]
    placeholder: bool
    equipable: bool
    equipable_by_player: bool
    equipable_weapon: bool
    cost: int
    lowalch: int
    highalch: int
    weight: Optional[float]
    buy_limit: Optional[int]
    quest_item: bool
    release_date: Optional[str]
    duplicate: bool
    examine: Optional[str]
    icon: str
    wiki_name: Optional[str]
    wiki_url: Optional[str]
    equipment: Optional[ItemEquipment] = None
    weapon: Optional[ItemWeapon] = None

    @classmethod
    def from_json(cls, json_dict: Dict) -> 'ItemProperties':
        """Construct ItemProperties object from dictionary/JSON."""
        # Convert the dictionary under the 'equipment' key into ItemEquipment.
        if json_dict.get("equipable_by_player"):
            equipment = json_dict.pop("equipment")
            json_dict["equipment"] = ItemEquipment(**equipment)

        # Convert the dictionary under the 'weapon' key into ItemWeapon.
        if json_dict.get("weapon"):
            weapon = json_dict.pop("weapon")
            json_dict["weapon"] = ItemWeapon(**weapon)

        return cls(**json_dict)

    def construct_json(self) -> Dict:
        """Construct dictionary for exporting or printing.

        :return: All class attributes stored in a dictionary.
        """
        return asdict(self)

    def export_json(self, pretty: bool, export_path: str):
        """Output ItemProperties to JSON file.

        :param pretty: Toggles pretty (indented) JSON output.
        :param export_path: The folder location to save the JSON output to.
        """
        json_out = self.construct_json()
        out_file_name = str(self.id) + ".json"
        out_file_path = Path(export_path / out_file_name)
        with open(out_file_path, "w") as out_file:
            if pretty:
                json.dump(json_out, out_file, indent=4)
            else:
                json.dump(json_out, out_file)



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Description:
Tests for module: osrsbox.monsters_api.all_monsters

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
import os
from pathlib import Path

from osrsbox.monsters_api import all_monsters

# The current number of monsters being loaded from the db
NUMBER_OF_MONSTERS = 2747


def test_all_monsters_load_monsters_json(path_to_docs_dir: Path):
    path_to_monsters_json_dir_no_slash = path_to_docs_dir / "monsters-json"
    path_to_monsters_json_dir_slash = os.path.join(path_to_docs_dir, "monsters-json", "")

    for path in (path_to_monsters_json_dir_slash, path_to_monsters_json_dir_no_slash):
        all_db_monsters = all_monsters.AllMonsters(str(path))
        assert len(all_db_monsters.all_monsters) == NUMBER_OF_MONSTERS


def test_all_monsters_load_monsters_complete(path_to_docs_dir: Path):
    path_to_monsters_complete = path_to_docs_dir / "monsters-complete.json"

    all_db_monsters = all_monsters.AllMonsters(str(path_to_monsters_complete))
    assert len(all_db_monsters.all_monsters) == NUMBER_OF_MONSTERS



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2020, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
from osrsbox.prayers_api import all_prayers


def load() -> all_prayers.AllPrayers:
    """Load the prayers database.

    :return all_prayers: An AllPrayers object containing the entire prayer database.
    """
    return all_prayers.AllPrayers()



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""

import json
from typing import Dict
from typing import List
from pathlib import Path
from dataclasses import asdict
from dataclasses import dataclass

from osrsbox.monsters_api.monster_drop import MonsterDrop


@dataclass
class MonsterProperties:
    """This class defines the object structure and properties for an OSRS monster.

    The MonsterProperties class is the object that retains all properties and stats
    for one specific monster. Every monster has the properties defined in this class,
    as well as the stats.
    """
    id: int = None
    name: str = None
    incomplete: bool = None
    members: bool = None
    release_date: str = None
    combat_level: int = None
    size: int = None
    hitpoints: int = None
    max_hit: int = None
    attack_type: str = None
    attack_speed: int = None
    aggressive: bool = None
    poisonous: bool = None
    immune_poison: bool = None
    immune_venom: bool = None
    attributes: List = None
    category: List = None
    slayer_monster: bool = None
    slayer_level: int = None
    slayer_xp: int = None
    slayer_masters: List = None
    duplicate: bool = None
    examine: str = None
    icon: str = None
    wiki_name: str = None
    wiki_url: str = None
    attack_level: int = None
    strength_level: int = None
    defence_level: int = None
    magic_level: int = None
    ranged_level: int = None
    attack_stab: int = None
    attack_slash: int = None
    attack_crush: int = None
    attack_magic: int = None
    attack_ranged: int = None
    defence_stab: int = None
    defence_slash: int = None
    defence_crush: int = None
    defence_magic: int = None
    defence_ranged: int = None
    attack_accuracy: int = None
    melee_strength: int = None
    ranged_strength: int = None
    magic_damage: int = None
    drops: Dict = None

    @classmethod
    def from_json(cls, json_dict: Dict) -> List[MonsterDrop]:
        """Convert the dict under the 'drops' key into actual :class:`MonsterDrop`"""
        monster_drops = list()
        if json_dict.get("drops"):
            for drop in json_dict["drops"]:
                monster_drops.append(MonsterDrop(**drop))

        json_dict["drops"] = monster_drops

        return cls(**json_dict)

    def construct_json(self) -> Dict:
        """Construct dictionary/JSON for exporting or printing.

        :return json_out: All class attributes stored in a dictionary.
        """
        return asdict(self)

    def export_json(self, pretty: bool, export_path: str):
        """Output Monster to JSON file.

        :param pretty: Toggles pretty (indented) JSON output.
        :param export_path: The folder location to save the JSON output to.
        """
        json_out = self.construct_json()
        out_file_name = str(self.id) + ".json"
        out_file_path = Path(export_path / out_file_name)
        with open(out_file_path, "w", newline="\n") as out_file:
            if pretty:
                json.dump(json_out, out_file, indent=4)
            else:
                json.dump(json_out, out_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib import Path
from setuptools import find_packages, setup

# Package meta-data.
NAME = 'osrsbox'
DESCRIPTION = 'A complete and up-to-date database of Old School Runescape (OSRS) items, monsters and prayers accessible using a Python API.'
URL = 'https://github.com/osrsbox/osrsbox-db'
EMAIL = 'phoil@osrsbox.com'
AUTHOR = 'PH01L'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = "1.3.20"

# Name of any third party packages that are required from the `osrsbox` package.
REQUIRED = [
    'dataclasses;python_version<"3.7"'
]

# Import the README and use it as the long-description.
readme_location = Path(__file__).parent
readme_location = Path(readme_location / "README.md")

with open(readme_location, encoding='utf-8') as f:
    long_description = '\n' + f.read()

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(),
    install_requires=REQUIRED,
    include_package_data=True,
    license='GPLv3',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
)



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2020, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
import json
from pathlib import Path
from typing import Dict, List, Union, Generator

from osrsbox.prayers_api.prayer_properties import PrayerProperties

PATH_TO_PRAYERS_COMPLETE_JSON = Path(__file__).absolute().parent / ".." / ".." / "docs" / "prayers-complete.json"
if not PATH_TO_PRAYERS_COMPLETE_JSON.is_file():
    PATH_TO_PRAYERS_COMPLETE_JSON = Path(__file__).absolute().parent / ".." / "docs" / "prayers-complete.json"
    if not PATH_TO_PRAYERS_COMPLETE_JSON.is_file():
        raise ValueError("Error: Default prayer database file not found. Exiting")


class AllPrayers:
    """This class handles loading of the osrsbox-db prayers database.

    :param input_data_file_or_directory: The osrsbox-db prayers folder of JSON files, or single JSON file.
    """
    def __init__(self, input_data_file_or_directory: Path = PATH_TO_PRAYERS_COMPLETE_JSON):
        self.all_prayers: List[PrayerProperties] = list()
        self.all_prayers_dict: Dict[int, PrayerProperties] = dict()
        self.load_all_prayers(input_data_file_or_directory)

    def __iter__(self) -> Generator[PrayerProperties, None, None]:
        """Iterate (loop) over each PrayerProperties object."""
        for prayer in self.all_prayers:
            yield prayer

    def __getitem__(self, id_number: int) -> PrayerProperties:
        """Return the prayer definition object for a loaded prayer.

        :param id_number: The prayer ID number.
        :return: The prayer definition object linked to a specific ID number.
        """
        return self.all_prayers_dict[id_number]

    def __len__(self) -> int:
        """Return the count of the total number of prayers.

        :return: The total number of prayers.
        """
        return len(self.all_prayers)

    def lookup_by_prayer_id(self, prayer_id_number: int) -> PrayerProperties:
        """Lookup a specific prayer ID and get the associated PrayerProperties object.

        :param prayer_id_number: The prayer ID to lookup.
        :return: The PrayerProperties object found from the lookup.
        :raises: KeyError when the prayer ID cannot be found.
        """
        try:
            prayer_properties = self.all_prayers_dict[prayer_id_number]
        except KeyError:
            raise KeyError("Cannot find the provided prayer ID number...")
        return prayer_properties

    def load_all_prayers(self, input_data_file_or_directory: Union[Path, str]) -> None:
        """Load the prayers database via a JSON file, or directory of JSON files.

        :param input_data_file_or_directory: The path to the data input.
        :raises ValueError: Valid input not found.
        """
        # Check if a str is supplied, if so, convert to Path object
        if isinstance(input_data_file_or_directory, str):
            input_data_file_or_directory = Path(input_data_file_or_directory)

        # Process the directory of JSON, or a single JSON file
        if input_data_file_or_directory.is_dir():
            self._load_prayers_from_directory(path_to_directory=input_data_file_or_directory)
        elif input_data_file_or_directory.is_file():
            self._load_prayers_from_file(path_to_json_file=input_data_file_or_directory)
        else:
            raise ValueError("Error: Valid input not found. Exiting.")

        # Sort the list of prayers
        self.all_prayers.sort(key=lambda x: x.id)

    def _load_prayers_from_directory(self, path_to_directory: Path) -> None:
        """Load prayer database from a directory of JSON files (`prayers-json`).

        :param path_to_directory: The path to the `prayers-json` directory.
        :raises ValueError: No JSON files found in supplied directory.
        """
        # Fetch all .json files in provided dir
        json_files = list(path_to_directory.glob("*.json"))

        try:
            json_files[0]
        except IndexError as e:
            raise ValueError("Error: No files found in directory, check the supplied path. Exiting.") from e

        # Loop through every prayer in JSON file
        for json_file in json_files:
            with open(json_file) as input_json_file:
                temp = json.load(input_json_file)

            self._load_prayer(temp)

    def _load_prayers_from_file(self, path_to_json_file: Path) -> None:
        """Load prayer database from a single JSON file (`prayers-complete.json`).

        :param path_to_json_file: The path to the `prayers-complete.json` file.
        """
        with open(path_to_json_file) as input_json_file:
            temp = json.load(input_json_file)

        for entry in temp:
            self._load_prayer(temp[entry])

    def _load_prayer(self, prayer_json: Dict) -> None:
        """Convert the `prayer_json` into a :class:`PrayerProperties` and store it.

        :param prayer_json: A dict from an open and loaded JSON file.
        :raises ValueError: Cannot populate prayer.
        """
        # Load the prayer using the PrayerProperties class
        try:
            prayer_def = PrayerProperties.from_json(prayer_json)
        except TypeError as e:
            raise ValueError("Error: Invalid JSON structure found, check supplied input. Exiting") from e

        # Add prayer to list
        self.all_prayers.append(prayer_def)
        self.all_prayers_dict[prayer_def.id] = prayer_def



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Copyright (c) 2019, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
from dataclasses import asdict
from dataclasses import dataclass
from typing import Dict
from typing import Optional


@dataclass
class ItemEquipment:
    """This class defines the properties for an equipable OSRS item.

    The ItemEquipment class is the object that retains all items properties related
    to equipable items. This includes item stats (attack, defence bonuses) and
    additional properties about equipment slot, attack speed and skill requirements
    for the item.
    """
    attack_stab: int
    attack_slash: int
    attack_crush: int
    attack_magic: int
    attack_ranged: int
    defence_stab: int
    defence_slash: int
    defence_crush: int
    defence_magic: int
    defence_ranged: int
    melee_strength: int
    ranged_strength: int
    magic_damage: int
    prayer: int
    slot: str
    requirements: Optional[Dict]

    def construct_json(self) -> Dict:
        """Construct dictionary/JSON of ItemEquipment class for exporting or printing.

        :return: All class attributes stored in a dictionary.
        """
        return asdict(self)



"""
Author:  PH01L
Email:   phoil@osrsbox.com
Website: https://www.osrsbox.com

Description:
Tests for module: docs/prayers-json

Copyright (c) 2020, PH01L

###############################################################################
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
"""
import json
from pathlib import Path

import config


def test_prayer_database():
    """Unit test to check prayer database contents against JSON schema."""
    # Read in the prayer schema file
    path_to_schema = Path(config.DATA_SCHEMAS_PATH / "schema-prayers.json")
    with open(path_to_schema, 'r') as f:
        schema = json.loads(f.read())

    # Validator object with schema attached
    v = config.MyValidator(schema)

    # Set the path to the prayers-json folder and get all the JSON files
    path_to_prayers_json_dir = Path(config.DOCS_PATH / "prayers-json")
    fis = path_to_prayers_json_dir.glob("*.json")
    fis = sorted(fis)

    # Validate each file
    for json_file in fis:
        with open(json_file) as fi:
            prayer = json.load(fi)
            # print(prayer["id"])
            assert v.validate(prayer)
